=== modified file 'NEWS'
--- NEWS	2006-09-21 17:00:00 +0000
+++ NEWS	2006-09-21 19:27:23 +0000
@@ -37,6 +37,9 @@
 
   BUG FIXES:
 
+    * Transform._set_mode() needs to stat the right file. 
+      (John Arbash Meinel, #56549)
+
     * Raise WeaveFormatError rather than StopIteration when trying to read
       an empty Weave file. (John Arbash Meinel, #46871)
 

=== modified file 'bzrlib/tests/test_transform.py'
--- bzrlib/tests/test_transform.py	2006-09-06 12:00:01 +0000
+++ bzrlib/tests/test_transform.py	2006-09-21 19:27:23 +0000
@@ -15,8 +15,13 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 import os
+import stat
+import sys
 
-from bzrlib import tests
+from bzrlib import (
+    tests,
+    urlutils,
+    )
 from bzrlib.bzrdir import BzrDir
 from bzrlib.conflicts import (DuplicateEntry, DuplicateID, MissingParent,
                               UnversionedParent, ParentLoop, DeletingParent,)
@@ -30,7 +35,7 @@
 from bzrlib.transform import (TreeTransform, ROOT_PARENT, FinalPaths, 
                               resolve_conflicts, cook_conflicts, 
                               find_interesting, build_tree, get_backup_name)
-import bzrlib.urlutils as urlutils
+
 
 class TestTreeTransform(TestCaseInTempDir):
 
@@ -535,6 +540,36 @@
         self.assertTrue(wt.is_executable('soc'))
         self.assertTrue(wt.is_executable('sac'))
 
+    def test__set_mode_stats_correctly(self):
+        """_set_mode stats to determine file mode."""
+        if sys.platform == 'win32':
+            raise TestSkipped('chmod has no effect on win32')
+
+        stat_paths = []
+        real_stat = os.stat
+        def instrumented_stat(path):
+            stat_paths.append(path)
+            return real_stat(path)
+
+        transform, root = self.get_transform()
+
+        bar1_id = transform.new_file('bar', root, 'bar contents 1\n',
+                                     file_id='bar-id-1', executable=False)
+        transform.apply()
+
+        transform, root = self.get_transform()
+        bar1_id = transform.trans_id_tree_path('bar')
+        bar2_id = transform.trans_id_tree_path('bar2')
+        try:
+            os.stat = instrumented_stat
+            transform.create_file('bar2 contents\n', bar2_id, mode_id=bar1_id)
+        finally:
+            os.stat = real_stat
+            transform.finalize()
+
+        bar1_abspath = self.wt.abspath('bar')
+        self.assertEqual([bar1_abspath], stat_paths)
+
 
 class TransformGroup(object):
     def __init__(self, dirname):
@@ -731,6 +766,7 @@
         self.assertIs(os.path.lexists(this.wt.abspath('b/h1.OTHER')), False)
         self.assertEqual(this.wt.id2path('i'), pathjoin('b/i1.OTHER'))
 
+
 class TestBuildTree(tests.TestCaseWithTransport):
 
     def test_build_tree(self):

=== modified file 'bzrlib/transform.py'
--- bzrlib/transform.py	2006-09-10 23:42:17 +0000
+++ bzrlib/transform.py	2006-09-21 19:27:23 +0000
@@ -292,7 +292,7 @@
         except KeyError:
             return
         try:
-            mode = os.stat(old_path).st_mode
+            mode = os.stat(self._tree.abspath(old_path)).st_mode
         except OSError, e:
             if e.errno == errno.ENOENT:
                 return

